USE [Database Name]
GO

/* ===============================================================
Base de donnes : 							 
Procdure stocke : 	exploitation.up_Add_Logdata_CpgLogdata_BO_ID
Commentaire : Ajoute les tables CPGDATA, CPGEVENTS du BO et leurs index dans schma de partition, et modifie ou cr la vue partitionne et les triggers
Cre le : 2017-08-31						 
Modifie le : 						 
Auteur : Jf pedrazzani
Paramtres : 	
	@BO_ID int,
	@PartitionSchemeName varchar(255)
	@LogdataViewName varchar(50)
	@CpgLogdataViewName varchar(50)

Version : 1.0.0								 
											 
Exemple dappel : 
	Exec exploitation.up_Add_Logdata_CpgLogdata_BO_ID 1582,  'ps_SoSqlLogDataByDays', 'v_SoSqlCpgData', 'v_SoSqlCpgEvents';
	Aprs renomage
	Exec exploitation.up_Add_Logdata_CpgLogdata_BO_ID 1582,  'ps_SoSqlLogDataByDays', 'SoSqlCpgData', 'SoSqlCpgEvents';
	
	Declare @BOID int = 1

	While @BOID <= 12
	Begin
		Exec exploitation.up_Add_Logdata_CpgLogdata_BO_ID @BOID,  'ps_SoSqlLogDataByDays', 'v_SoSqlCpgData', 'v_SoSqlCpgEvents';
		Set @BOID += 1;
	End

Cas :	remplacement de tables par des vues partitionnes
Frquence : N/A
===============================================================*/
Create or Alter procedure [exploitation].[up_Add_Logdata_CpgLogdata_BO_ID]
(
	@BO_ID int,
	@PartitionSchemeName varchar(255),
	@LogdataViewName varchar(50),
	@CpgLogdataViewName varchar(50),
	@PrintOnly bit = 1
)
As
BEGIN

	Declare @SoSqlCpgEventsShortTableName varchar(255) = 'dbo.t_SoSqlCpgEvents_CHECKED_FOR_';
	Declare @SoSqlCpgDataShortTableName varchar(255) = 'dbo.t_SoSqlCpgData_CHECKED_FOR_';

	Declare @SoSqlCpgEventsTableName varchar(255) = @SoSqlCpgEventsShortTableName+Convert(varchar(50), @BO_ID);
	Declare @SoSqlCpgDataTableName varchar(255) = @SoSqlCpgDataShortTableName+Convert(varchar(50), @BO_ID);
	Declare @SqlCmd varchar(4000) = '';

	IF not Exists(Select top 1 1 from sys.partition_schemes where name = @PartitionSchemeName)
	Begin
		Raiserror('The specified partition scheme does not exists!', 16, 10)
		return;
	End
	
	IF object_id(@SoSqlCpgEventsTableName) is null
	Begin
		Set @SqlCmd = 'CREATE TABLE '+@SoSqlCpgEventsTableName+'(
			[LOG_ID] [bigint] IDENTITY(1,1) NOT NULL,
			[MD5_ID] [bigint] NULL,
			[CPG_ID] [int] NOT NULL,
			[PID_ID] [int] NULL,
			[CID_ID] [int] NULL,
			[PID_ID_DATA] [int] NULL,
			[CID_ID_DATA] [int] NULL,
			[PID_ID_LINK] [int] NULL,
			[CID_ID_LINK] [int] NULL,
			[CPG_ID_EVENT] [int] NULL,
			[PID_ID_EVENT] [int] NULL,
			[CID_ID_EVENT] [int] NULL,
			[LOG_CREATIONDATE] [datetime] NOT NULL DEFAULT (getdate()),
			[LOG_EXTDATE] [date] NOT NULL DEFAULT (getdate()),
			[bo_id] [int] NOT NULL CHECK (bo_id = '+Convert(varchar(50), @BO_ID)+'),
			[MD5Type_ID] [int] NULL,
			CONSTRAINT [PK_'+@SoSqlCpgEventsTableName+'] PRIMARY KEY CLUSTERED 
				(
					LOG_ID, LOG_EXTDATE
				)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, DATA_COMPRESSION = PAGE)ON ['+@PartitionSchemeName+']([LOG_EXTDATE])
		) ON ['+@PartitionSchemeName+']([LOG_EXTDATE])
		'
		--Cr la table du BO
		Exec(@SqlCmd)

		--Ajout des indexes dans le partition scheme
		IF object_id(@SoSqlCpgEventsTableName) is not null
		Begin
			Set @SqlCmd = 'CREATE NONCLUSTERED INDEX [IXD_CPGLOGDATA_MD5ID] ON '+@SoSqlCpgEventsTableName+'
			(
				[MD5_ID] ASC
			)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, DATA_COMPRESSION = PAGE) ON ['+@PartitionSchemeName+']([LOG_EXTDATE])'
	
			Exec(@SqlCmd)

			Set @SqlCmd = 'CREATE NONCLUSTERED INDEX [IX_CPGLOGDATA_CPG_ID_PID_ID_LINK_BO_ID] ON '+@SoSqlCpgEventsTableName+'
			(
				[CPG_ID] ASC,
				[PID_ID_LINK] ASC
			)
			INCLUDE ([LOG_ID],
				[MD5_ID],
				[PID_ID]) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, DATA_COMPRESSION = PAGE) ON ['+@PartitionSchemeName+']([LOG_EXTDATE])'
	
			Exec(@SqlCmd)

			Set @SqlCmd = 'CREATE NONCLUSTERED INDEX [IX_CPGLOGDATA_CPGID] ON '+@SoSqlCpgEventsTableName+'
			(
				[CPG_ID] ASC,
				[LOG_EXTDATE] ASC
			)
			INCLUDE ([MD5_ID],
				[PID_ID],
				[CID_ID]) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, DATA_COMPRESSION = PAGE) ON ['+@PartitionSchemeName+']([LOG_EXTDATE])'
	
			Exec(@SqlCmd)

			Set @SqlCmd = 'CREATE NONCLUSTERED INDEX [IX_ID_LINK] ON '+@SoSqlCpgEventsTableName+'
			(
				[PID_ID_LINK] ASC
			)
			INCLUDE ([LOG_ID]) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, DATA_COMPRESSION = PAGE) ON ['+@PartitionSchemeName+']([LOG_EXTDATE])'
	
			Exec(@SqlCmd)

			Set @SqlCmd = 'CREATE NONCLUSTERED INDEX [IX_MD5_ID] ON '+@SoSqlCpgEventsTableName+'
			(
				[MD5_ID] ASC
			)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, DATA_COMPRESSION = PAGE) ON ['+@PartitionSchemeName+']([LOG_EXTDATE])'
	
			Exec(@SqlCmd)

			Set @SqlCmd = 'CREATE NONCLUSTERED INDEX [IX_PID_ID_LINK_CID_ID_LINK_BO_ID] ON '+@SoSqlCpgEventsTableName+'
			(
				[PID_ID_LINK] ASC,
				[CID_ID_LINK] ASC
			)
			INCLUDE ([MD5_ID],
				[CPG_ID],
				[PID_ID],
				[CID_ID]) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, DATA_COMPRESSION = PAGE) ON ['+@PartitionSchemeName+']([LOG_EXTDATE])'
	
			Exec(@SqlCmd)
		End
	End



	IF object_id(@SoSqlCpgDataTableName) is null
	Begin
		Set @SqlCmd = 'CREATE TABLE '+@SoSqlCpgDataTableName+'(
			[LOG_ID] [bigint] IDENTITY(1,1) NOT NULL,
			[BO_ID] [int] NOT NULL CHECK (BO_ID = '+Convert(varchar(50), @BO_ID)+'),
			[COOKIES_ID] [bigint] NULL,
			[MD5_ID] [bigint] NULL,
			[PID_ID] [int] NULL,
			[CID_ID] [int] NULL,
			[LOG_CREATIONDATE] [datetime] NOT NULL DEFAULT (getdate()),
			[LOG_EXTDATE] [date] NOT NULL DEFAULT (getdate()),
			[MD5Type_ID] [int] NULL,
			CONSTRAINT [PK_'+@SoSqlCpgDataTableName+'] PRIMARY KEY CLUSTERED 
			(
				LOG_ID ASC, LOG_EXTDATE
			)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, DATA_COMPRESSION = PAGE)ON ['+@PartitionSchemeName+']([LOG_EXTDATE])
		) ON ['+@PartitionSchemeName+']([LOG_EXTDATE])
		'
		--Cr la table du BO
		Exec(@SqlCmd)

		--Ajoute les index dans le partition scheme
		Set @SqlCmd = 'CREATE NONCLUSTERED INDEX [IDX_MD5_ID] ON '+@SoSqlCpgDataTableName+'
		(
			[MD5_ID] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, DATA_COMPRESSION = PAGE) ON ['+@PartitionSchemeName+']([LOG_EXTDATE])'
		Exec(@SqlCmd)
	End


	Declare @ViewDefinition varchar(4000) = '';
	Declare @CreateAlterView varchar(4000) = '';

	--Ajout/update les vues
	IF object_id(@SoSqlCpgEventsTableName) is not null
	Begin
		Select --*,
		@ViewDefinition += Case
			When @ViewDefinition = '' Then 'Select * from '+table_name
			Else char(13)+'Union All'+char(13)+'Select * from '+table_name
		End 
		from information_schema.tables
		where (table_schema+'.'+table_name) like (@SoSqlCpgEventsShortTableName+'%')

		IF Exists(select top 1 1 from sys.all_objects where type = 'v' and schema_name(schema_id) = 'dbo' and name = @CpgLogdataViewName)
		Begin
			Set @CreateAlterView = 'Alter view dbo.'+@CpgLogdataViewName+char(13)+'as'+char(13)+@ViewDefinition
		End
		ELSE
		Begin
			Set @CreateAlterView = 'Create view dbo.'+@CpgLogdataViewName+char(13)+'as'+char(13)+@ViewDefinition
		End

		Exec(@CreateAlterView);
	End

	Set @ViewDefinition = '';
	Set @CreateAlterView = '';
	IF object_id(@SoSqlCpgDataTableName) is not null
	Begin
		Select --*,
		@ViewDefinition += Case
			When @ViewDefinition = '' Then 'Select * from '+table_name
			Else char(13)+'Union All'+char(13)+'Select * from '+table_name
		End 
		from information_schema.tables
		where (table_schema+'.'+table_name) like (@SoSqlCpgDataShortTableName+'%')

		IF Exists(select top 1 1 from sys.all_objects where type = 'v' and schema_name(schema_id) = 'dbo' and name = @LogdataViewName)		
		Begin
			Set @CreateAlterView = 'Alter view dbo.'+@LogdataViewName+char(13)+'as'+char(13)+@ViewDefinition
		End
		ELSE
		Begin
			Set @CreateAlterView = 'Create view dbo.'+@LogdataViewName+char(13)+'as'+char(13)+@ViewDefinition
		End

		Exec(@CreateAlterView);
	End

	Declare @TriggerDefinition varchar(max) = '';
	Declare @CreateAlterTrigger varchar(max) = '';

	--Ajout/update les vues
	IF 1 = 1 --object_id(@SoSqlCpgEventsTableName) is not null
	Begin
		Select --*,
		@TriggerDefinition +=  '
	if Exists(Select top 1 1 from inserted where bo_id = '+Reverse(Left(Reverse(table_name), charindex('_', Reverse(table_name))-1))+')
	Begin
		INSERT INTO [dbo].['+table_name+'](MD5_ID, CPG_ID, PID_ID, CID_ID, PID_ID_DATA, CID_ID_DATA, PID_ID_LINK, CID_ID_LINK, CPG_ID_EVENT, PID_ID_EVENT, CID_ID_EVENT, LOG_CREATIONDATE, LOG_EXTDATE, bo_id, MD5Type_ID)
		SELECT MD5_ID, CPG_ID, PID_ID, CID_ID, PID_ID_DATA, CID_ID_DATA, PID_ID_LINK, CID_ID_LINK, CPG_ID_EVENT, PID_ID_EVENT, CID_ID_EVENT, LOG_CREATIONDATE, LOG_EXTDATE, bo_id, MD5Type_ID
		FROM inserted
		Where bo_id = '+Reverse(Left(Reverse(table_name), charindex('_', Reverse(table_name))-1))+'
	End'+char(13)

		from information_schema.tables
		where (table_schema+'.'+table_name) like (@SoSqlCpgEventsShortTableName+'%')

		IF Exists(select top 1 1 from sys.all_objects where type = 'v' and schema_name(schema_id) = 'dbo' and name = @CpgLogdataViewName)
		Begin
			IF Exists(SELECT top 1 1 FROM sys.triggers WHERE Name = 'trig_InsteadOfInsertCpgEvents')
			Begin
				Set @CreateAlterTrigger = 'Alter TRIGGER trig_InsteadOfInsertCpgEvents on dbo.'+@CpgLogdataViewName+char(13)+' INSTEAD OF INSERT'+char(13)+'as'+char(13)+'BEGIN'+char(13)+@TriggerDefinition+char(13)+'END'
			End
			ELSE
			Begin
				Set @CreateAlterTrigger = 'Create TRIGGER trig_InsteadOfInsertCpgEvents on dbo.'+@CpgLogdataViewName+char(13)+' INSTEAD OF INSERT'+char(13)+'as'+char(13)+'BEGIN'+char(13)+@TriggerDefinition+char(13)+'END'
			End
		End

		Exec(@CreateAlterTrigger);
	End

	Set @TriggerDefinition = '';
	Set @CreateAlterTrigger = '';

	--Ajout/update les vues
	IF 1 = 1 --object_id(@SoSqlCpgEventsTableName) is not null
	Begin
		Select --*,
		@TriggerDefinition +=  '
	if Exists(Select top 1 1 from inserted where bo_id = '+Reverse(Left(Reverse(table_name), charindex('_', Reverse(table_name))-1))+')
	Begin
		INSERT INTO [dbo].['+table_name+'](BO_ID, COOKIES_ID, MD5_ID, PID_ID, CID_ID, LOG_CREATIONDATE, LOG_EXTDATE, MD5Type_ID)
		SELECT BO_ID, COOKIES_ID, MD5_ID, PID_ID, CID_ID, LOG_CREATIONDATE, LOG_EXTDATE, MD5Type_ID
		FROM inserted
		Where bo_id = '+Reverse(Left(Reverse(table_name), charindex('_', Reverse(table_name))-1))+'
	End'+char(13)

		from information_schema.tables
		where (table_schema+'.'+table_name) like (@SoSqlCpgDataShortTableName+'%')

		IF Exists(select top 1 1 from sys.all_objects where type = 'v' and schema_name(schema_id) = 'dbo' and name = @LogdataViewName)
		Begin
			IF Exists(SELECT 1 FROM sys.triggers WHERE Name = 'trig_InsteadOfInsertCpgData')
			Begin
				Set @CreateAlterTrigger = 'Alter TRIGGER trig_InsteadOfInsertCpgData on dbo.'+@LogdataViewName+char(13)+' INSTEAD OF INSERT'+char(13)+'as'+char(13)+'BEGIN'+char(13)+@TriggerDefinition+char(13)+'END'
			End
			ELSE
			Begin
				Set @CreateAlterTrigger = 'Create TRIGGER trig_InsteadOfInsertCpgData on dbo.'+@LogdataViewName+char(13)+' INSTEAD OF INSERT'+char(13)+'as'+char(13)+'BEGIN'+char(13)+@TriggerDefinition+char(13)+'END'
			End
		End

		Exec(@CreateAlterTrigger);
	End

END